classdef IRISDrill <handle
    % Creates and accesses an IRIS.WellboreSimulator.Drilling.DrillingCalculator object.
    %
    % Prerequisites:
    % * Matlab version >=2009a
    % * Requisite IRIS dll's must be installed.
    % * Non-dotNet IRIS dll's must lie in the system path (Wemod.dll,
    % DrillStringMech.dll and WellboreModel.dll). They are normally located
    % at J:/commonlib/dll/win32/. To add them to the system path, go to
    % Start Menu in WIndows, and select Settings->Control
    % Panel->System->Advanced, click on Environment Variables button at
    % bottom, select 'Path' from list of System Variables, and append the
    % dll-path at the end.
    % * IRISDrill.depot property must be set to the directory of the
    %   IRIS dotNet dll's. Default value is J:/commonlib/dll/dotNet. 
    % * gacutil must be installed. This is done simplest by installing
    %   Microsoft Visual Studio Express C# (freeware)
    %
    %
    % A IRISDrill object is simply created with
    %  > md=IRISDrill();
    % It requires a wellbore object to run, and can load a stored
    % binary-format wellbore file:
    %   > md.loadWellbore(filename);
    % This command also does necessary household work like initializing
    % various log structures.
    % At this point it is possible to change any of the default properties
    % IRISDrill passes on to the DrillingCalculator object.
    % It is also possible to modify the Wellbore object, e.g. change the
    % properties of its associated Reservoir object.
    % More detailed documentation about this wil hopefully appear...
    % The DrillingCalculator can now be started:
    %   > md.initialize();
    % Now, the methods and properties of the DrillingCalculator object are
    % easily accessible through the calc property of md:
    %   > md.calc.Step()
    %   > disp( md.calc.CalculatedSPP )
    % DrillingCalculator returns profiles in a rather complicated format.
    % IRISDrill contains a (static) utility method to convert them into
    % arrays:
    %   > p=md.getProfile(md.calc.GetAnnulusPressureProfile());
    % p is now a 2*profile_length array, with p(1,:) containing depth and
    % p(2,:) values.
    %
    % For further information about the methods and properties of the
    % DrillingCalculator object, refer to the commonlib documentation.
    %
    % As of version 0.2, running registerDLL/deregisterDLL is done
    % automatically, so they do no longer need do be explicitly invoked.
    
    
    properties
        wellbore;
        calc;
        initialGelDepth=0;
        lastCirculationDepth;
        UseSteadyStateHydraulicModel = false;
        tenMinutes = System.TimeSpan(0, 10, 0);
        hasFloatValve = true;
        hasAutoFillAnnulus = true;
        hasChoke = false;
        useTemperatureModel = true;
        useGelModel = false;
        durationSinceLastCirculation = 0;
        slopeLimiterType=1;
        useAutoConnection=true;
    end
    properties(Constant)
        version=0.21;
    end
    properties(Constant, SetAccess=private)
        depot='C:\IRIS\commonlib\dll\dotNet'
       
        dlls={'IRIS.Serialization',...
            'IRIS.Compression',...
            'IRIS.Field',...
            'IRIS.Facade.Wemod',...
            'IRIS.Math',...
            'IRIS.Physics.ContinuousMedium',...
            'IRIS.RealTimeParameter',...
            'IRIS.Organization',...
            'IRIS.Statistics',...
            'IRIS.Std',...
            'IRIS.Model.Drilling',...
            'IRIS.Well',...
            'IRIS.WellboreData',...
            'IRIS.LM',...
            'IRIS.Facade.DrillStringMech',...
            'IRIS.WellboreData.Remoting',...
            'IRIS.WellboreSimulator.Drilling'}

        gacutil='C:\Program Files\Microsoft SDKs\Windows\v7.0A\bin\gacutil'
        
        
    end
    methods
        function this=IRISDrill()
            this.registerDLL(this.dlls);
            this.calc=IRIS.WellboreSimulator.Drilling.DrillingCalculator();
            this.deregisterDLL(this.dlls);
        end
        function this=loadWellbore(this,fname)
            serializer=IRIS.Serialization.Serializer();
            this.wellbore=serializer.BinaryLoader(fname);
        end
        function this=initialize(this)
            dsr = this.wellbore.StringRuns.Last;
            this.calc.CalculatedLog = dsr.CalculatedValuesLog;
            this.calc.HydraulicAnnulusLog = dsr.HydraulicAnnulusLog;
            this.calc.HydraulicStringLog = dsr.HydraulicStringLog;
            this.calc.TorqueDragLog = dsr.TorqueAndDragLog;
            
            this.lastCirculationDepth = this.wellbore.GetLastCirculationDepth(System.DateTime.Now);
            if isempty(dsr.RealtimeLog)
                dsr.RealtimeLog = IRIS.WellboreData.LogCreator.CreateDrillingRawDataRealtimeLog();
            end
            if isempty(dsr.CalculatedValuesLog)
                dsr.CalculatedValuesLog = IRIS.WellboreData.LogCreator.CreateCalculatedValuesLog();
            end
            if isempty(dsr.HydraulicStringLog)
                dsr.HydraulicStringLog = IRIS.WellboreData.LogCreator.CreateHydraulicStringLog();
            end
            if isempty(dsr.HydraulicAnnulusLog)
                dsr.HydraulicAnnulusLog = IRIS.WellboreData.LogCreator.CreateHydraulicAnnulusLog();
            end
            if isempty(dsr.TorqueAndDragLog)
                dsr.TorqueAndDragLog = IRIS.WellboreData.LogCreator.CreateTorqueAndDragLog();
            end
            this.calc.Log = dsr.RealtimeLog;
            this.calc.CalculatedLog = dsr.CalculatedValuesLog;
            this.calc.HydraulicStringLog = dsr.HydraulicStringLog;
            this.calc.HydraulicAnnulusLog = dsr.HydraulicAnnulusLog;
            this.calc.TorqueDragLog = dsr.TorqueAndDragLog;
            this.calc.TorqueDragModelType = IRIS.WellboreSimulator.Drilling.TorqueDragModelType.CSharp;
            this.calc.UseMechanicalModel = false;
            this.calc.Initialize(this.wellbore, ...
                System.DateTime.Now.Subtract(this.tenMinutes), ...
                this.durationSinceLastCirculation, ...
                this.initialGelDepth, ...
                this.calc.LiquidLevelString, ...
                this.calc.LiquidLevelAnnulus, ...
                this.hasFloatValve, ...
                this.hasAutoFillAnnulus, ...
                this.useTemperatureModel, ...
                this.useGelModel, ...
                this.slopeLimiterType, ...
                this.useAutoConnection);
        end
    end
    
    methods(Static)
        function r=getProfile(p)
            c=p.Left.Count;
            r=zeros(2,c);
            for a=1:c
                r(1,c)= p.Left.Item(a-1);
                r(2,c)= p.Right.Item(a-1);
            end
        end
        function registerDLL(d)
            silent='/silent';
            if iscell(d)
            for a=1:length(d)
                system(sprintf('"%s" %s /i %s\\%s.dll',IRISDrill.gacutil,silent,IRISDrill.depot,d{a}));
                disp(d{a})
            end
            for a=1:length(d)
                NET.addAssembly(d{a});
            end
            else
                system(sprintf('"%s" %s /i %s\\%s.dll',IRISDrill.gacutil,silent,IRISDrill.depot,d));
                NET.addAssembly(d);
            end
        end
        function deregisterDLL(d)
            silent='/silent';
            if iscell(d)
            for a=1:length(d)
                system(sprintf('"%s" %s /u %s',IRISDrill.gacutil,silent,d{a}));
            end
            else
                system(sprintf('"%s" %s /u %s',IRISDrill.gacutil,silent,d));
            end
        end
    end
    
end