%%This file runs the heave compensation case on the IRISDrill simulator
%%using the internal model controller without feedback linearization for 
%%bottom hole pressure control.
%%Written in 2011 by Ingar Landet
clc
clear all;

%% Initialize variables
Tstop= 2000;
FlowrateIn=zeros(Tstop,1);
AnnulusChokeOpening=zeros(Tstop,1);
ChokeLineChokeOpening=zeros(Tstop,1);
AnnulusPressure=zeros(Tstop,1);
ChokeLinePressure=zeros(Tstop,1);
ChokeLinePressure2 = zeros(Tstop,1);
StandpipePressure=zeros(Tstop,1);
BottomholePressure=zeros(Tstop,1);
BottomholePressure2 = zeros(Tstop,1);
AnnulusFlowrateOut=zeros(Tstop,1);
ChokeLineFlowrateOut=zeros(Tstop,1);
DensityIn=zeros(Tstop,1);
DensityOut=zeros(Tstop,1);
DensityAnnulus=zeros(Tstop,1);
ChokePressure= zeros(Tstop,1);
TopOfStringVelocity=zeros(Tstop,1);
qbpp = zeros(Tstop,1);
dT=0.05;
t=1.00;
F_run=true;
counter=1.00;

%% Initialize drillingcalculator
md=IRISDrill();
md.loadWellbore('U2262bar.binaryWellbore'); %Ullrigg well U2 data
md.initialize();
md.calc.StepDuration = dT;
md.calc.ChokeOpening = 1;
md.calc.ChokeLineChokeOpening = 1;
md.calc.FlowRateIn=0/60000;  % u1
md.calc.ChokePumpFlowRateIn=3000/60000; % u3 Flowrate for Choke Line Pump 
md.calc.DensityIn=1475; % u4
md.calc.ChokePumpDensityIn=1475; % u6 - choke pump fluid density  [kg/m3]
md.calc.Step();
md.calc.ChokeOpening = 0.3; % Control Choke, used as control input
md.calc.ChokeLineChokeOpening = 10^-4;  % Kill line choke, not used
t_cpu=0;
n = 1;



%% Initialize IMC controller
Q0 = 0.7*10^-8;
Q1 = 0.2*10^-7;
s = tf('s');
p_ref = 262*10^5;
% Use 1 single frequency
cSys = Q1 + Q1/s + Q0*s/(s^2+(pi/6)^2);

% Initial conditions for the controller
xW = [0 0 0];
array = zeros(1,100);
for i = 1:99
    t = [0:0.5:1];
    e = 10^6*(-0.3-array(i));
    [Y,t,X] = lsim(ss(cSys),e*ones(size(t)),t,xW);
    xW = X(end,:);
    array(i+1) = Y(end);
end

% Choke characteristics
Z_c = (1/100)*[0 4 9 10 15 20 30 40 50 70]';
G_c = 1/700*[0 1 35 45 80 125 200 300 400 700]';
g_cInv=@(g) interp1( G_c, Z_c, max(0, min(g, 1)) );

%% Run to stabilize pressures
md.calc.StepDuration = 1;
for i = 1:300
    md.calc.Step();
end
%Run control case
md.calc.StepDuration = dT;
while F_run
    tic	% Start timer
    % Move drillstring
    md.calc.TopOfStringVelocity = -2*pi/12*sin(2*pi*(n+i)*dT/12);
    % Run controllers
    t = [0 dT];
    e = p_ref-md.calc.CalculatedDownholePressure;
    [Y,t,X] = lsim(ss(cSys),e*ones(size(t)),t,xW);
    xW = X(end,:);
    r =-Y(end);
    md.calc.ChokeOpening = g_cInv(r);

    % Step simulator
    md.calc.Step();

    % Log variables
    qbpp(n) = md.calc.ChokePumpFlowRateIn;
    StandpipePressure(n)=md.calc.PressureChokePump;
    TopOfStringVelocity(n) = md.calc.TopOfStringVelocity;
    FlowrateIn(n)=md.calc.FlowRateIn;
    AnnulusChokeOpening(n)=md.calc.ChokeOpening;
    ChokeLineChokeOpening(n)=md.calc.ChokeLineChokeOpening;
    DensityIn(n)=md.calc.DensityIn;
    DensityOut(n)=md.calc.DensityOut;
    DensityAnnulus(n)=md.calc.DensityAnnulus;
    AnnulusPressure(n)=md.calc.CalculatedChokePressure;
    ChokeLinePressure(n)=md.calc.GetSensorValue('ChokeLineSensor_1','Measure Point Pressure');
    ChokeLinePressure2(n)=md.calc.GetSensorValue('DrillstringSensor_E2880','Measure Point Pressure');
    StandpipePressure(n)=md.calc.CalculatedSPP;
    BottomholePressure(n)=md.calc.CalculatedDownholePressure;
    BottomholePressure2(n)=md.calc.GetSensorValue('DrillstringSensor_E1','Measure Point Pressure');
    AnnulusFlowrateOut(n)=md.calc.CalculatedFlowRateOut;
    ChokeLineFlowrateOut(n)=-md.calc.GetSensorValue('ChokeLineSensor_1','Measure Point Total Flowrate');
    
    


    t = t + dT;	% Update time stamp for next step
    % Pause remainder of sample time
    t_cpu = toc; % Computation time
%     pause(dT - t_cpu);
    if mod(n,100) == 0
       n/Tstop 
    end
    if n>Tstop
        F_run = false;
    end
    n = n+1;

end

%% Do some plotting
t = 0:dT:Tstop*dT;
subplot(2,2,1);
plot(t,BottomholePressure,'k');
hold on
plot(t,262*10^5*ones(size(t)),'k--');
xlabel('time [s]');
ylabel('p_b_i_t [Pa]');
subplot(2,2,2);
plot(t,AnnulusPressure,'k');
hold on
xlabel('time [s]');
ylabel('p_c [Pa]');
subplot(2,2,3);
plot(t,AnnulusChokeOpening,'k');
xlabel('time [s]');
ylabel('u_c [0-1]');
subplot(2,2,4);
plot(t,cos(2*pi*(t+300*dT)/12),'k');
xlabel('time [s]');
ylabel('x_d [m]');




