function [Geometry] = findAreasAndVolumes(P)
%findAreasAndVolumes  
%
%   Returns the well and drill string geoemtries
%
%   [Geometry] = findAreasAndVolumes(P)
%       P   Parameters; struct with specific field names
%       Geometry    Parameteres; struct with specific field names
%           
%
% Authors:
%   Ingar Landet <ilan@statoil.com>            (IL)
%
% Revision history:
%  2010/07/15  IL      v1.0	Created
%  2010/12/14  IL      v1.1 Changed to meet model changes from fall project
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


V_a0  = P.V_a0;         % [m3]      Annulus volume without drill string
L     = P.L;            % [m]       Total length of well
L_p = P.L_p;            % [m]       Lengths with different diameter drill strings
D_dI = P.D_dI;          % [m]       Inner diameters different diameter drill strings
D_dO = P.D_dO;          % [m]       Outer diameters different diameter drill strings
N = P.N;                %           Number of cells on each side of the drill bit
l = L/N;


%Initialize arrays
A_d = zeros(N,1);
A_a = zeros(N,1);
dh_a = zeros(N,1);
dh_d = zeros(N,1);

%Calculate entire volumes
n = length(D_dI);
V_d = 0;
for i=1:n,
    V_d = V_d + pi/4*D_dI(i)^2*L_p(i); 
end 

V_a = V_a0;
n = length(D_dO);
for i=1:n,
    V_a = V_a - pi/4*( D_dO(i)^2 )*L_p(i); 
end

%Fill up length and cross section areas
%If new data for the well annulus is available, this part would have to be
%changed
%There is no smartness in choosing control volumes, so this should be done
%manually
A_d(1) = (pi/4)*D_dI(2)^2;
l_d = l*ones(N,1);
A_d(2:N) = (pi/4)*D_dI(1)^2*ones(N-1,1);

A_a(1) = V_a0/L - (pi/4)*D_dO(2)^2;
l_a = l*ones(N,1);
A_a(2:N) =V_a0/L - (pi/4)*D_dO(1)^2*ones(N-1,1);


%Calculate the height difference of each cell
x1 = 0;
x2 = l;
dh_d(1) = h(x2) - h(x1);
x2 = l;
dh_a(1) = h(x2) - h(x1);
for i =2:N
   x1 = sum(l_d(1:i-1));
   x2 = sum(l_d(1:i));
   dh_d(i) = h(x2) - h(x1);
   x1 = sum(l_a(1:i-1));
   x2 = sum(l_a(1:i));
   dh_a(i) = h(x2) - h(x1);
end

%Function to calculate the height of a point x
function height = h(x)
    if x < 800
        height = x;
    else
        height = 800 + (750/1190)*(x-800);
    end
end

%Rearrange arrays to be consistent with flow direction
A_a = flipud(A_a);
l_a = flipud(l_a);
dh_a = flipud(dh_a);

Geometry.D_dI = D_dI(2)*ones(N,1);
Geometry.D_dI(1) = D_dI(1);
Geometry.D_dO = D_dO(2)*ones(N,1);
Geometry.D_dO(1) = D_dO(1);
Geometry.A_a = A_a;
Geometry.l_a = l_a;
Geometry.dh_a = dh_a;
Geometry.A_d = A_d;
Geometry.l_d = l_d;
Geometry.dh_d = dh_d;
Geometry.V_d = V_d;
Geometry.V_a = V_a;

end